/*****************************************************************************************
 *
 * Part of "Go to menu"
 * Copyright (c) 2006, 2007, 2010 NuukiaWorld
 *
 *  "Go to menu" is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  Version    Date          Author             Log
 *  --------------------------------------------------------------------------------------
 *  1.00       19.02.2006    NuukiaWorld        Initial Release
 *  1.02       26.02.2006    NuukiaWorld        New organized "Add link" menu
 *  1.03       21.01.2007    NuukiaWorld        SDK 2.41
 *  1.08       01.07.2010    NuukiaWorld        Fixed global_buffer overflow
 *  1.09       15.10.2010    NuukiaWorld        Size optimizations, removed heap usage
 *
 ****************************************************************************************/

#include "main.h"

/**************************************************************************************/

static void create_group_menu(void) {
   struct menu_item buf;
   unsigned int i;
   char *item;

   menu_init_item(&buf);
   buf.msg = OMSG_SELECT_GROUP;
   buf.action = action_select_back;
   item = (char*)menu_main.items + menu_item_size;
   for( i = 1; i <= menu_main.count; i++ ) {
      buf.title = *(nokstr_t**)item;
      item += menu_item_size;
      buf.arg = i;
      menu_add_item( 2, 0, &buf );
   }
}

static void create_link_menu( unsigned int group ) {
   struct menu_item buf;

   clear_menu1();
   buf.msg = OMSG_SELECT_LINK;
   buf.action = action_add_back;
   buf.flags = MIF_SHOW_DONE;
   enum_menu( group << 8,
      (struct static_menu_item*)( (char*)menu_main.items + menu_item_size * group ), &buf );
}

/**************************************************************************************/

/* UI messages dispatcher */
extern rexx_uint add_dispatcher_id; /* dispatcher's call id */
rexx("add_dispatcher_id=add_5e0_id(add_dispatcher)"); /* auto creation of the id */

unsigned int add_dispatcher( unsigned int msg ) {

   switch(msg) {
      case MSG_D_INIT: // application initialization
         if( load_links() >= MAX_LINKS ) {
            free(app.links);
            display_msg( DMI_INFORMATION, &str_menu_full );
            break;
         }
         menu_create( 2, MDM_SUB | MDM_LIST, MCM_MARK_ID, 1, &str_add_link );
         menu_create( 1, MDM_SUB | MDM_LIST, MCM_MARK_ID, 1, &str_add_link );
         create_group_menu();
         menu_open( 2, 1 );
         msg = MSG_R_STAY;
         break;

      case MSG_D_QUIT: // handle application quit
         menu_delete(1);
         menu_delete(2);
         free(app.links);
         msg = MSG_R_QUIT;
         break;

      case OMSG_SELECT_GROUP:
         create_link_menu(message.argv[0]);
         menu_open( 1, 1 );
         msg = MSG_NOP;
         break;

      case OMSG_SELECT_LINK:
         app.links[app.count] = message.argv[0];
         save_links();
         send_message(MSG_REQUEST_QUIT);
         msg = MSG_NOP;
         break;
   }
   return msg;
}

/**************************************************************************************/

