/*****************************************************************************************
 *
 * Part of "Go to menu"
 * Copyright (c) 2006, 2007, 2010 NuukiaWorld
 *
 *  "Go to menu" is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  Version    Date          Author             Log
 *  --------------------------------------------------------------------------------------
 *  1.00       19.02.2006    NuukiaWorld        Initial Release
 *  1.05       31.12.2007    NuukiaWorld        Changed fill_goto_menu(), added popup
 *  1.08       01.07.2010    NuukiaWorld        Fixed global_buffer overflow
 *  1.09       15.10.2010    NuukiaWorld        Size optimizations, removed heap usage
 *
 ****************************************************************************************/

#include "main.h"

/**************************************************************************************/

/* Move selected item in specified direction (-1, +1) in the menu
*/
static void move_item( int dir ) {

   struct menu_item *item1, *item2;
   nokstr_t *temp;
   unsigned int old, new;
   

   if( app.source < 0 ) return;              // No item selected yet

   old = ( app.dest < 0 ? app.source : app.dest ) + 1;

   new = old + dir;
   if( new == 0 ) new = app.count;
   if( new > app.count ) new = 1;

   // Move menu item titles
   item1 = menu_get_item( 1, old );
   temp = item1->title;
   
   while( old < new ) {
      item2 = menu_get_item( 1, ++old );
      item1->title = item2->title;
      item1 = item2;
   }
   while( old > new ) {
      item2 = menu_get_item( 1, --old );
      item1->title = item2->title;
      item1 = item2;
   }

   item1->title = temp;
   item1->action = action_move_cancel;
   item1->flags = MIF_SHOW_DONE;
   app.dest = new - 1;
}

/**************************************************************************************/

/* UI messages dispatcher */
extern rexx_uint edit_dispatcher_id; /* dispatcher's call id */
rexx("edit_dispatcher_id=add_5e0_id(edit_dispatcher)"); /* auto creation of the id */

unsigned int edit_dispatcher( unsigned int msg ) {

   struct menu_item *item;
   unsigned int i, temp;

   switch(msg) {
      case MSG_D_INIT: // application initialization
         if( load_links() == 0 ) {
            free(app.links);
            display_msg( DMI_INFORMATION, &str_menu_empty );
            break;
         }
         menu_create( 1, MDM_SUB | MDM_LIST | MDM_DISABLE_SHORTCUTS, MCM_MARK_ID | MCM_OWN_COUNTER,
            1, NULL );
         fill_goto_menu(1);
         app.save = false;
         app.source = app.dest = -1;
         menu_open( 1, 1 );
         msg = MSG_R_STAY;
         break;

      case MSG_D_QUIT: // handle application quit
         menu_delete(1);
         if(app.save) save_links();
         free(app.links);
         msg = MSG_R_QUIT;
         break;

      case OMSG_SELECT_LINK:
         msg = MSG_NOP;
         if( app.count == 1 ) {
            app.source = 0;   // No need to select the link
         } else if( app.source < 0 ) {
            // Link selected
            item = menu_get_item( 1, message.argv[0] + 1 );
            item->action = action_remove_cancel;
            item->flags = MIF_SHOW_DONE;
            app.source = message.argv[0];
            display_msg( DMI_INFORMATION, &str_move_or_erase );
            menu_reopen();
            break;
         }
         
         // Something will be changed
         app.save = true;
         temp = app.links[app.source];
         
         if( app.dest < 0 ) {
            // Link deleted
            app.dest = --app.count;
            temp = 0;
         }
         
         // Move links
         for( i = app.source; i < app.dest; i++ ) app.links[i] = app.links[i + 1];
         for( ; i > app.dest; i-- ) app.links[i] = app.links[i - 1];
         app.links[app.dest] = temp;
         
         // FALL_THROUGH
      case MSG_BACK:
         if( app.source < 0 ) break;   // No item selected
         clear_menu1();
         fill_goto_menu(1);
         app.source = app.dest = -1;
         menu_reopen();
         msg = MSG_NOP;
         break;
         
      case MSG_CLOSE_DISPLAY_MESSAGE:
         if( app.count == 0 ) {
            // All links deleted, quit
            send_message( MSG_DISPLAY_MESSAGE | MSG_2P, DMI_INFORMATION, &str_menu_empty );
            send_message(MSG_REQUEST_QUIT);
            break;
         }
         break;
         
      case MSG_KEY_UP:
         move_item(-1);
         break;
         
      case MSG_KEY_DOWN:
         move_item(+1);
         break;
   }
   return msg;
}

/**************************************************************************************/

