/*****************************************************************************************
 *
 * Part of "Go to menu"
 * Copyright (c) 2006, 2007, 2008, 2010 NuukiaWorld
 *
 *  "Go to menu" is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  Version    Date          Author             Log
 *  --------------------------------------------------------------------------------------
 *  1.00       19.02.2006    NuukiaWorld        Initial Release
 *  1.01       23.02.2006    NuukiaWorld        Hook code moved to hook.s
 *  1.05       31.12.2007    NuukiaWorld        Changed id to MID_GOTO, improved closing
 *  1.06       01.02.2008    NuukiaWorld        Check of msg_argv[0] at MSG_D_QUIT_NOW
 *  1.08       01.07.2010    NuukiaWorld        Fixed global_buffer overflow
 *  1.09       15.10.2010    NuukiaWorld        Size optimizations, removed heap usage
 *
 ****************************************************************************************/

#include "main.h"

/**************************************************************************************/

/* UI messages dispatcher */
extern rexx_uint goto_dispatcher_id; /* dispatcher's call id */
rexx("goto_dispatcher_id=add_5e0_id(goto_dispatcher)"); /* auto creation of the id */

unsigned int goto_dispatcher( unsigned int msg ) {

   struct menu_item *link;

   switch(msg) {
      case MSG_D_INIT: // application initialization
         if( load_links() == 0 ) {
            display_msg( DMI_INFORMATION, &str_menu_empty );
            break;
         }
         menu_create( MID_GOTO, MDM_SUB | MDM_LIST | MDM_ENABLE_SHORTCUTS, MCM_MARK_ID, 1, NULL );
         if( fill_goto_menu(MID_GOTO) == 1 ) {
            // Skip menu if it contains only 1 link
            link = menu_get_item( MID_GOTO, 1 );
            send_message( link->msg | MSG_1P, link->arg );
            // Alarm clock dialog does not appear without this:
            send_message( MSG_MENU_SHORTCUT_INFO | MSG_2P, link->arg, 0 );
         } else {
            menu_open( MID_GOTO, 1 );
         }
         msg = MSG_R_STAY;
         break;

      case MSG_D_QUIT_NOW: // returning from another application (app not valid anymore)
         if( msg_argv[0] != QNS_OK && menu_get_item( MID_GOTO, 2 ) != NULL ) {
            // Reopen Go to menu if it contains more than 1 link
            menu_reopen();
            msg = MSG_R_STAY;
            break;
         }
         // FALL_THROUGH
      case MSG_D_QUIT:     // handle application quit
         menu_reopen();    // ensure that the menu will be closed, even after shortcut
         menu_close();
         menu_delete(MID_GOTO);
         msg = MSG_R_QUIT;
         break;
   }
   return msg;
}

/**************************************************************************************/

