/*****************************************************************************************
 *
 * Part of "Go to menu"
 * Copyright (c) 2006, 2007, 2010 NuukiaWorld
 *
 *  "Go to menu" is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  Version    Date          Author             Log
 *  --------------------------------------------------------------------------------------
 *  1.00       19.02.2006    NuukiaWorld        Initial Release
 *  1.02       26.02.2006    NuukiaWorld        New organization for link ids
 *  1.03       21.01.2007    NuukiaWorld        SDK 2.41
 *  1.05       31.12.2007    NuukiaWorld        Added 'id' in fill_goto_menu()
 *  1.08       01.07.2010    NuukiaWorld        Fixed global_buffer overflow
 *  1.09       15.10.2010    NuukiaWorld        Size optimizations, removed heap usage
 *
 ****************************************************************************************/

#include "main.h"

/**************************************************************************************/

/* Loads links from EEPROM to app.links and sets app.count accordingly.
   After success, the caller is responsible for freeing app.links.
   Returns true for success.
*/
unsigned int load_links(void) {

   unsigned int i;

   eeprom_read( app.links, eprid_links, EPM_NORMAL );
   for( i = 0; i < MAX_LINKS; i++ ) {
      // Check the validity of each link!
      if( app.links[i] >> 8 == 0 || app.links[i] >> 8 > menu_main.count
         || ( app.links[i] & 0xFF ) == 0 ) break;
   }
   app.count = i;
   return i;
}

void clear_menu1(void) {
	while( menu_get_item( 1, 1 ) != NULL ) menu_rem_item( 1, 1 );
}

/* Enumerates a static menu and its siblings. Two modes:
   n < 0:   Increments n for each menu item and stops when id reaches zero.
            The corresponding menu item is copied to buf. ("Find target")
   n >= 0:  Increments n for each menu item and adds the item title and current
            n value as an new dynamic menu item. ("List targets")
   Returns the resulting n value.
*/
int enum_menu( int n, const struct static_menu_item *item, struct menu_item *buf ) {

   unsigned int i, msg, arg;
   bool_t games;
   char* p;

   buf->title = item->title;
   p = (char*)item + menu_item_size;
   msg = *(unsigned short*)( p - 10 );
   arg = *(unsigned int*)( p - 16 );
   if( ++n > 0 ) {
      // "List targets" mode
      buf->arg = n;
      menu_add_item( 1, 0, buf );
   } else {
      if( msg == MSG_OWN ) msg = MSG_START_HANDLER;   // NokiX games
      buf->msg = msg;
      buf->arg = arg;
      buf->flags = *(unsigned char*)( p - 4 );
   }
   
   games = ( msg == MSG_START_HANDLER && arg == games_5e0_id );
   if(games) {
      // Include Nokix games in the games group
      msg = MSG_OPEN_STATIC_MENU;
      arg = (unsigned int)&menu_games;
   }
   
   if( msg == MSG_OPEN_STATIC_MENU && ( ( (struct menu*)arg )->confmode & 0x0C ) == 0 ) {
      // Submenu without config_id settings, dump its items also
      p = (char*)( (struct menu*)arg )->items;
      for( i = 0; n != 0 && i < ( (struct menu*)arg )->count; i++ ) {
         p += menu_item_size;
         msg = *(unsigned short*)( p + menu_item_size - 10 );
         if( !games || msg == MSG_OWN || msg == MSG_OPEN_STATIC_MENU ) {
            // In the games menu, include only NokiX games and the settings item
            n = enum_menu( n, (struct static_menu_item*)p, buf );
         }
      }
   }
   return n;
}

/* Fill the dynamic menu #id with all defined links.
   id == MID_GOTO:   Launching Goto menu, default msg/arg/flags
   id != MID_GOTO:   Editing Goto menu, predefined msg/arg/flags
*/
unsigned int fill_goto_menu( unsigned int id ) {

   struct menu_item buf;
   unsigned int i;

   menu_init_item(&buf);
   buf.action = action_select_back;
   for( i = 0; i < app.count; i++ ) {

      // Get link data => buf
      enum_menu( -( app.links[i] & 0xFF ),
         (struct static_menu_item*)( (char*)menu_main.items
         + menu_item_size * ( app.links[i] >> 8 ) ), &buf );
      
      if( id != MID_GOTO ) {
         // Edit mode, change item properties
         buf.msg = OMSG_SELECT_LINK;
         buf.arg = i;
         if( app.count == 1 ) {   // No need to select the link
            buf.action = action_remove_cancel;
            buf.flags = MIF_SHOW_DONE;
         } else {
            buf.flags = MIF_FINISH_SHORTCUT;
         }
      }
      menu_add_item( id, 0, &buf );
   }
   return i;
}

