#ifndef MAIN_H
#define MAIN_H

/*****************************************************************************************
 *
 * Part of "Go to menu"
 * Copyright (c) 2006, 2007 NuukiaWorld
 *
 *  "Go to menu" is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *  Version    Date          Author             Log
 *  --------------------------------------------------------------------------------------
 *  1.00       19.02.2006    NuukiaWorld        Initial Release
 *  1.01       23.02.2006    NuukiaWorld        Some extern definitions removed
 *  1.02       01.03.2006    NuukiaWorld        (see other files)
 *  1.03       21.01.2007    NuukiaWorld        SDK 2.41
 *  1.05       31.12.2007    NuukiaWorld        (see other files)
 *  1.08       01.07.2010    NuukiaWorld        Fixed global_buffer overflow
 *  1.09       15.10.2010    NuukiaWorld        Size optimizations, removed heap use
 *
 ****************************************************************************************/

#include <nokix/all.h>

/*******************************************************/

/* Define 32-bit custom types that are efficient for ARM7 registers.
   Use inside functions and in function prototypes when there is no
   need for truncation to a specific number of bits!
*/
typedef  unsigned int char_t;  // Use instead of char
typedef  unsigned int bool_t;  // Use instead of bool

/**************************************************************************************/

#define  MAX_LINKS   9

/* KEEP THIS STRUCTURE <=40 BYTES LONG! */
struct application
{
   unsigned short    links[MAX_LINKS]; // HI=group (1...menu_main.count), LO=link (1...)
   unsigned char     count;
   unsigned char     save;
   signed char       source;
   signed char       dest;
};

/* global_buffer is an array of 40 bytes in RAM which you can use for your
   own purposes as long as your application stays active; when it is deactivated
   the buffer may be overwritten by another one;
   here we define 'app' symbol to access these 40 bytes as a structure */
#define app (*(struct application *)global_buffer)

/**************************************************************************************/

extern   unsigned int   load_links(void);
#define  save_links()   eeprom_write( app.links, eprid_links, EPM_NORMAL )

extern   void        clear_menu1(void);
extern   int         enum_menu( int n, const struct static_menu_item *item, struct menu_item *buf );
extern   unsigned int   fill_goto_menu( unsigned int id );

extern   const struct menu menu_main;
extern   const struct menu menu_games;

extern   const nokstr_t str_add_link;
extern   const nokstr_t str_menu_full;
extern   const nokstr_t str_menu_empty;
extern   const nokstr_t str_move_or_erase;

extern   rexx_uint   games_5e0_id;

extern   rexx_uint   action_add_back;
extern   rexx_uint   action_remove_cancel;
extern   rexx_uint   action_move_cancel;

extern   rexx_uint   eprid_links;

#define  OMSG_SELECT_LINK  MSG_OWN
#define  OMSG_SELECT_GROUP ( MSG_OWN + 1 )

#define  MID_GOTO    255   // Same dynamic menu ID should not be used anywhere in the phone

#endif /* MAIN_H */

